# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2019-02-20 14:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0001_initial'),
    ]


    operations = [
        migrations.AddField(
            model_name='notification',
            name='image',
            field=models.ImageField(
                upload_to='notification_imgs',
                null=True,
                blank=True,
                help_text="Image to accompany notification (optional)",
            )
        ),
    ]
